/*
this file is part of Silenda v0.1 
- Copyright © Alexander Weinhold, 2010

Silenda is a simple command line tool for hiding ASCII-Strings in
(uncompressed) Bitmaps and retrieving them again.

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, version 2 of the License.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


#include <stdio.h>
#include <string.h>
#include "silenda.h"

/* fastForward [times] bytes and copy them to output file */
void fastForward(FILE *ifp, FILE *ofp, int times) 
{
	int i;	
	for (i=0; i<times; i++) {
		fputc(fgetc(ifp),ofp);
	}
}

/* hide the characters in the bytes of ifp, put out to
 * ofp (start at current pointer position of ifp) */
void pushIntoFile(FILE *ifp, FILE *ofp, char* string) 
{
	char letter;
	unsigned char tempByte;
	do {
		/* 1000 0000, the 1 will get shifted around to extract each bit of the letter */			
		unsigned char conjunctor=0x80;
 
		/* get current character */
		letter = *string;

		while (conjunctor != 0) {
			tempByte = fgetc(ifp); 
			if ((letter & conjunctor) != 0)
				/*set last Bit*/
				tempByte = tempByte | 0x01;
			else
				/*unset last Bit*/
				tempByte = tempByte & 0xFE;
			fputc(tempByte, ofp);
			conjunctor = conjunctor >> 1;
		}

		/* set pointer to next character */
		string++;			
	} while (letter != '\0'); /* Strings always end with binary zero */
}

void copyUntilEnd(FILE *ifp, FILE *ofp) 
{
	while(!feof(ifp)) {
		fputc(fgetc(ifp),ofp);
	}
}

long int getFileSize(FILE *file) 
{
	fseek (file,0,SEEK_END);
	long int fileSize = ftell(file);
	rewind(file);
	return fileSize;
}

int main(int argc, char *argv[]) 
{
	if (argc != 4)
		end("Usage: encoder inputFile outputFile \"Text\"");

	
	
	FILE *ifp;
	FILE *ofp;
	char* string = argv[3];
	int pictureOffset;
		
	printf("Trying to push \"%s\" into data of %s\n" , string, argv[1]);

	if ((ifp = fopen(argv[1], "rb"))!=NULL && (ofp = fopen(argv[2],"wb"))!= NULL) {
		
		pictureOffset = getPictureOffset(ifp);
		long int usableBytes = getFileSize(ifp) - pictureOffset;
		long int pushableLetters;
		printf("Data-Offset: %d Bytes\n",pictureOffset);
		printf("Usable Bytes: %ld Bytes",usableBytes);
		printf(" (Corresponds to %ld letters)\n", pushableLetters=usableBytes / 8);
		printf("Length of String: %lu letters (+1 for end mark)\n", strlen(string));

		/* 
		 * we are going to push the '\0' that ends the string into the file as well
		 * so we need room to store it (thus "+1")
		 */
		if (strlen(string)+1 > pushableLetters)
			end("We don't have enough room to store the string");

		/* fastForward to where actual picture data is stored */
		fastForward(ifp,ofp,pictureOffset);

		pushIntoFile(ifp,ofp,string);
		
		copyUntilEnd(ifp,ofp);

		fclose(ifp);
		fclose(ofp);

		return 0;
	}
	else
		end("Error opening file(s)");
}
